// Generated by CoffeeScript 2.3.1
(function() {
  var $, ChildProcess, Files, Fs, Groups, Handlebars, defaults, disabled, echo, expandPath, generate, nop, reload, reloadNginx, server_name, server_port, setDisabled, setFile, setReload, setTemplate, sync, toConfig, verbose, warn, writeNginxFile;

  ({$, echo, warn, verbose} = require('../common'));

  Fs = require('fs');

  Files = require('../files');

  ({Groups} = require('./groups'));

  Handlebars = require('handlebars');

  ChildProcess = require('child_process');

  ({expandPath} = Files);

  defaults = {
    reload: reload = "service nginx reload",
    server_name: server_name = null,
    server_port: server_port = 80,
    disabled: disabled = true,
    template: Files.nginxTemplate = "%/nginx.template",
    templateText: "upstream {{ name }} {\n{{#each group}}\n{{#if this.port}}\n	server 127.0.0.1:{{ this.port }} {{#unless this.started}}down{{/unless}};\n{{/if}}\n{{/each}}\n}\nserver {\n	listen {{ public_port }};\n	server_name {{ public_name }};\n	location / {\n		proxy_pass http://{{ name }};\n	}\n}\n{{#if ssl_cert}}\nserver {\n	listen 443;\n	server_name {{ public_name }};\n	ssl on;\n	ssl_certificate {{ ssl_cert }};\n	ssl_certificate_key {{ ssl_key }};\n	location / {\n		proxy_pass http://{{ name }};\n	}\n}\n{{/if}}"
  };

  setReload = function(r) {
    return reload = r;
  };

  setTemplate = function(f) {
    return Files.nginxTemplate = f;
  };

  setFile = function(f) {
    return Files.nginxFile = f.replace(/^~/, process.env.HOME).replace(/^%/, Files.basePath);
  };

  setDisabled = function(d) {
    return disabled = d;
  };

  generate = (t) => {
    var buf;
    buf = "";
    if (t == null) {
      return "no template";
    }
    Groups.forEach((group) => {
      var ref, ref1;
      if (group.port != null) {
        return buf += t({
          group: group,
          public_name: (ref = group.public_name) != null ? ref : group.name,
          public_port: (ref1 = group.public_port) != null ? ref1 : 80,
          ssl_cert: group.ssl_cert,
          ssl_key: group.ssl_key,
          name: group.name
        });
      }
    });
    return buf;
  };

  nop = function() {};

  writeNginxFile = (cb) => {
    var start;
    cb || (cb = nop);
    if (disabled) {
      return cb(null, false);
    }
    echo(`Saving nginx file: ${expandPath(Files.nginxFile)}...`);
    start = Date.now();
    verbose("Reading nginx template...", expandPath(Files.nginxTemplate));
    Fs.readFile(expandPath(Files.nginxTemplate), function(err, data) {
      var text;
      if (err) {
        warn("Failed to read nginx template:", err);
        return typeof cb === "function" ? cb(null, false) : void 0;
      }
      text = generate(Handlebars.compile(data.toString('utf8')));
      verbose("Writing nginx file...", expandPath(Files.nginxFile));
      Fs.writeFile(expandPath(Files.nginxFile), text, function(err) {
        verbose(`writeNginxFile took ${Date.now() - start} ms`);
        return cb(err, err == null);
      });
      return null;
    });
    return null;
  };

  toConfig = () => {
    var _disabled, _file, _reload, _template, buf;
    _reload = reload === defaults.reload ? "" : ` --reload-cmd '${reload}'`;
    _file = Files.nginxFile === Files.makePath("nginx") ? "" : ` --file '${Files.nginxFile}'`;
    _template = Files.nginxTemplate === defaults.template ? "" : ` --template '${Files.nginxTemplate}'`;
    _disabled = disabled ? " --disable" : " --enable";
    buf = `nginx${_disabled}${_reload}${_file}${_template}`;
    Groups.forEach((group) => {
      if ((group.public_name != null) || (group.public_port != null) || (group.ssl_cert != null) || (group.ssl_key != null)) {
        return buf += `\nnginx --group ${group.name}` + (group.public_name ? ` --name ${group.public_name}` : "") + (group.public_port ? ` --port ${group.public_port}` : "") + (group.ssl_cert ? ` --ssl_cert ${group.ssl_cert}` : "") + (group.ssl_key ? ` --ssl_key ${group.ssl_key}` : "");
      }
    });
    return buf;
  };

  reloadNginx = $.debounce(500, (cb) => {
    var p;
    cb || (cb = nop);
    if (disabled) {
      return cb(null, false);
    }
    echo("Reloading nginx...");
    p = ChildProcess.exec(reload, {
      shell: true
    });
    p.stdout.on('data', function(data) {
      return echo(`${reload}`, data.toString("utf8"));
    });
    p.stderr.on('data', function(data) {
      return echo(`${reload} (stderr)`, data.toString("utf8"));
    });
    return cb(null, false);
  });

  sync = (cb) => {
    return writeNginxFile((err, acted) => {
      if (err || !acted) {
        return typeof cb === "function" ? cb(err, acted) : void 0;
      }
      return reloadNginx(cb);
    });
  };

  Object.assign(module.exports, {writeNginxFile, reloadNginx, setReload, setTemplate, setFile, setDisabled, toConfig, sync, defaults});

}).call(this);
